==> btp-6.3.1.tar.gz: btp-6.3.1/rc.cxx <==
/*
  
  This file is from the xrc package, and has undergone some changes
  in order to be able to be compiled into btp. Most notably, main()
  was rendered almost unrecognizable and renamed to rcsend().  /Sam

  rc -  application for sending IR-codes via lirc
  
  Copyright (C) 1998 Christoph Bartelmus (lirc@bartelmus.de)
  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  
*/

#define LIRCD "/dev/lircd" //Probably shouldn't be hard coded.

#ifdef HAVE_CONFIG_H
#  include "config.h"
#endif

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <fcntl.h>
#include <getopt.h>
#include <sys/socket.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/un.h>
#include <errno.h>
#include <signal.h>
#include <limits.h>

#define PACKET_SIZE 256
/* three seconds */
#define TIMEOUT 3

int btperror(char *text, int fatal=0);
int timeout=0;
char *progname;

void sigalrm(int sig)
{
	timeout=1;
}

const char *read_string(int fd)
{
	static char buffer[PACKET_SIZE+1]="";
	char *end;
	static int ptr=0,end_len=0;
	ssize_t ret;
		
	if(ptr>0)
	{
		memmove(buffer,buffer+ptr,strlen(buffer+ptr)+1);
		ptr=strlen(buffer);
		end=strchr(buffer,'\n');
	}
	else
	{
		end=NULL;
	}
	alarm(TIMEOUT);
	while(end==NULL)
	{
		if(PACKET_SIZE<=ptr)
		{
			btperror("Bad packet");
			ptr=0;
			return(NULL);
		}
		ret=read(fd,buffer+ptr,PACKET_SIZE-ptr);

		if(ret<=0 || timeout)
		{
			if(timeout)
			{
				btperror("Timeout");
			}
			else
			{
				alarm(0);
			}
			ptr=0;
			return(NULL);
		}
		buffer[ptr+ret]=0;
		ptr=strlen(buffer);
		end=strchr(buffer,'\n');
	}
	alarm(0);timeout=0;

	end[0]=0;
	ptr=strlen(buffer)+1;
#       ifdef DEBUG
	printf("buffer: -%s-\n",buffer);
#       endif
	return(buffer);
}

enum packet_state
{
	P_BEGIN,
	P_MESSAGE,
	P_STATUS,
	P_DATA,
	P_N,
	P_DATA_N,
	P_END
};

int send_packet(int fd,const char *packet)
{
	int done,todo;
	const char *string,*data;
	char *endptr;
	enum packet_state state;
	int status,n;
	unsigned long data_n;

	todo=strlen(packet);
	data=packet;
	while(todo>0)
	{
		done=write(fd,(void *) data,todo);
		if(done<0)
		{
			btperror("Could not send packet");
			//perror(progname);
			return(-1);
		}
		data+=done;
		todo-=done;
	}

	/* get response */
	status=0;
	state=P_BEGIN;
	n=0;
	while(1)
	{
		string=read_string(fd);
		if(string==NULL) return(-1);
		switch(state)
		{
		case P_BEGIN:
			if(strcasecmp(string,"BEGIN")!=0)
			{
				continue;
			}
			state=P_MESSAGE;
			break;
		case P_MESSAGE:
			if(strncasecmp(string,packet,strlen(string))!=0 ||
			   strlen(string)+1!=strlen(packet))
			{
				state=P_BEGIN;
				continue;
			}
			state=P_STATUS;
			break;
		case P_STATUS:
			if(strcasecmp(string,"SUCCESS")==0)
			{
				status=0;
			}
			else if(strcasecmp(string,"END")==0)
			{
				status=0;
				return(status);
			}
			else if(strcasecmp(string,"ERROR")==0)
			{
			  //Fix this
			  //				btperror("%s: command failed: %s",
			  //					progname,packet);
       			        btperror("packet");
				status=-1;
			}
			else
			{
				goto bad_packet;
			}
			state=P_DATA;
			break;
		case P_DATA:
			if(strcasecmp(string,"END")==0)
			{
				return(status);
			}
			else if(strcasecmp(string,"DATA")==0)
			{
				state=P_N;
				break;
			}
			goto bad_packet;
		case P_N:
			errno=0;
			data_n=strtoul(string,&endptr,0);
			if(!*string || *endptr)
			{
				goto bad_packet;
			}
			if(data_n==0)
			{
				state=P_END;
			}
			else
			{
				state=P_DATA_N;
			}
			break;
		case P_DATA_N:
			btperror("P_DATA_N");
			// Fix this
			//			btperror(string);
			n++;
			if(n==data_n) state=P_END;
			break;
		case P_END:
			if(strcasecmp(string,"END")==0)
			{
				return(status);
			}
			goto bad_packet;
			break;
		}
	}
 bad_packet:
	btperror("Bad return packet");
	return(-1);
}

int valsend(int fd, char *command) {
  char buffer[PACKET_SIZE+1];

  if(strlen(command)<PACKET_SIZE) {
    sprintf(buffer,"SEND_ONCE %s\n",command);
    
    if(send_packet(fd,buffer)==-1) {
      btperror("Could not send\ncommand");
      close(fd);
      return(EXIT_FAILURE);
    }
  } else {
    btperror("Input too long");
    close(fd);
    return(EXIT_FAILURE);
  }
  return(EXIT_SUCCESS);
}

int rcsend(char *command, int fd)
{
	struct sockaddr_un addr;
	//	int fd;
	char buffer[PACKET_SIZE+1];
        char scratch[40];
	char remote[40];
	char macropart[40];
	int remote_set;
	char *scratchptr;
	char *cindex;
	struct sigaction act;
	int sendstatus;
/* use this */
//	progname=argv[0];
//      progname="btp";

	// Should this be run only once, too?

	remote_set = 0;
       	act.sa_handler=sigalrm;
       	sigemptyset(&act.sa_mask);
       	act.sa_flags=0;           /* we need EINTR */
       	sigaction(SIGALRM,&act,NULL);

	//       	addr.sun_family=AF_UNIX;
	//	strcpy(addr.sun_path,LIRCD);
	//	fd=socket(AF_UNIX,SOCK_STREAM,0);
	//	if(fd==-1)
	//	{
	//		btperror("Could not open\nsocket\n\nIs lircd running?");
	//		// I know, but I really don't like applications just dying.
	//		//		perror(progname);
	//		//		exit(EXIT_FAILURE);
	//		return(EXIT_FAILURE);
	//	};
	//	if(connect(fd,(struct sockaddr *)&addr,sizeof(addr))==-1)
	//	{
	//		btperror("Could not connect\nto socket\n\nIs lircd running?");
	//		// Same goes here.
	//		//		perror(progname);
	//		//		exit(EXIT_FAILURE);
	//		return(EXIT_FAILURE);
	//	};

	if(!strncasecmp(command, "macro ", 6)) {
	  //Ok... I really hate string juggling.
	  //I miss those neat array tossing and automatic enumerator thingies.
	  //Lessee, split on comma, then colon. Or something like that.
	  strcpy(buffer, command);
	  scratchptr = strtok(buffer, " "); //Skip "macro ", init strtok.
	  while(scratchptr = strtok(NULL, ",\n")) {
	    strcpy(scratch, scratchptr);
	    if(cindex = index(scratch, ':')) {
	      cindex[0] = 0;
	      if(!strncasecmp(scratch, ".sleep", 6)) {
		sleep(atoi(cindex+1));
	      } else if(!strncasecmp(scratch, ".tenths", 7)) {
		usleep(atoi(cindex+1)*100000);
	      } else if(!strncasecmp(scratch, ".exit", 5)) {
		printf("%s\n", cindex+1);
		exit(EXIT_SUCCESS); //There must be a better way.
	      } else { //Not a command, but a remote.
		strcpy(remote, scratch);
		remote_set = 1;
		sprintf(macropart,"%s %s", scratch, cindex+1);
		sendstatus = valsend(fd, macropart);
		if(sendstatus == EXIT_FAILURE) {
		  return(EXIT_FAILURE);
		}
	      }
	    } else {
	      if(remote_set) {
		sprintf(macropart,"%s %s", remote, scratch);
		sendstatus = valsend(fd, macropart);
		if(sendstatus == EXIT_FAILURE) {
		  return(EXIT_FAILURE);
		}
	      } else {
		btperror("Macro error!\nMissing remote\ndefinition.");
		return(EXIT_FAILURE);
	      }
	    }
	  }
	  //Phew! Macros! And delays!
	} else {
	  sendstatus = valsend(fd, command);
	}
	//	close(fd);
	return(sendstatus);
}

==> btp-6.3.2.tar.gz: btp-6.3.2/rc.cxx <==
/*
  
  This file is from the xrc package, and has undergone some changes
  in order to be able to be compiled into btp. Most notably, main()
  was rendered almost unrecognizable and renamed to rcsend().  /Sam

  rc -  application for sending IR-codes via lirc
  
  Copyright (C) 1998 Christoph Bartelmus (lirc@bartelmus.de)
  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
  
*/

#define LIRCD "/dev/lircd" //Probably shouldn't be hard coded.

#ifdef HAVE_CONFIG_H
#  include "config.h"
#endif

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <fcntl.h>
#include <getopt.h>
#include <sys/socket.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/un.h>
#include <errno.h>
#include <signal.h>
#include <limits.h>

#define PACKET_SIZE 256
/* three seconds */
#define TIMEOUT 3

int btperror(char *text, int fatal=0);
int timeout=0;
char *progname;

void sigalrm(int sig)
{
	timeout=1;
}

const char *read_string(int fd)
{
	static char buffer[PACKET_SIZE+1]="";
	char *end;
	static int ptr=0; 
///     static int end_len=0;
	ssize_t ret;
		
	if(ptr>0)
	{
		memmove(buffer,buffer+ptr,strlen(buffer+ptr)+1);
		ptr=strlen(buffer);
		end=strchr(buffer,'\n');
	}
	else
	{
		end=NULL;
	}
	alarm(TIMEOUT);
	while(end==NULL)
	{
		if(PACKET_SIZE<=ptr)
		{
			btperror("Bad packet");
			ptr=0;
			return(NULL);
		}
		ret=read(fd,buffer+ptr,PACKET_SIZE-ptr);

		if(ret<=0 || timeout)
		{
			if(timeout)
			{
				btperror("Timeout");
			}
			else
			{
				alarm(0);
			}
			ptr=0;
			return(NULL);
		}
		buffer[ptr+ret]=0;
		ptr=strlen(buffer);
		end=strchr(buffer,'\n');
	}
	alarm(0);timeout=0;

	end[0]=0;
	ptr=strlen(buffer)+1;
#       ifdef DEBUG
	printf("buffer: -%s-\n",buffer);
#       endif
	return(buffer);
}

enum packet_state
{
	P_BEGIN,
	P_MESSAGE,
	P_STATUS,
	P_DATA,
	P_N,
	P_DATA_N,
	P_END
};

int send_packet(int fd,const char *packet)
{
	int done,todo;
	const char *string,*data;
	char *endptr;
	enum packet_state state;
	int status,n;
	unsigned long data_n;

	todo=strlen(packet);
	data=packet;
	while(todo>0)
	{
		done=write(fd,(void *) data,todo);
		if(done<0)
		{
			btperror("Could not send packet");
			//perror(progname);
			return(-1);
		}
		data+=done;
		todo-=done;
	}

	/* get response */
	status=0;
	state=P_BEGIN;
	n=0;
	while(1)
	{
		string=read_string(fd);
		if(string==NULL) return(-1);
		switch(state)
		{
		case P_BEGIN:
			if(strcasecmp(string,"BEGIN")!=0)
			{
				continue;
			}
			state=P_MESSAGE;
			break;
		case P_MESSAGE:
			if(strncasecmp(string,packet,strlen(string))!=0 ||
			   strlen(string)+1!=strlen(packet))
			{
				state=P_BEGIN;
				continue;
			}
			state=P_STATUS;
			break;
		case P_STATUS:
			if(strcasecmp(string,"SUCCESS")==0)
			{
				status=0;
			}
			else if(strcasecmp(string,"END")==0)
			{
				status=0;
				return(status);
			}
			else if(strcasecmp(string,"ERROR")==0)
			{
			  // Fix this
			  //	btperror("%s: command failed: %s",
			  //	                      progname,packet);
       			        btperror("packet");
				status=-1;
			}
			else
			{
				goto bad_packet;
			}
			state=P_DATA;
			break;
		case P_DATA:
			if(strcasecmp(string,"END")==0)
			{
				return(status);
			}
			else if(strcasecmp(string,"DATA")==0)
			{
				state=P_N;
				break;
			}
			goto bad_packet;
		case P_N:
			errno=0;
			data_n=strtoul(string,&endptr,0);
			if(!*string || *endptr)
			{
				goto bad_packet;
			}
			if(data_n==0)
			{
				state=P_END;
			}
			else
			{
				state=P_DATA_N;
			}
			break;
		case P_DATA_N:
			btperror("P_DATA_N");
			// Fix this
			//			btperror(string);
			n++;
///                     if(n==data_n) state=P_END;
			if((unsigned)n==data_n) state=P_END;
			break;
		case P_END:
			if(strcasecmp(string,"END")==0)
			{
				return(status);
			}
			goto bad_packet;
			break;
		}
	}
 bad_packet:
	btperror("Bad return packet");
	return(-1);
}

int valsend(int fd, char *command) {
  char buffer[PACKET_SIZE+1];

  if(strlen(command)<PACKET_SIZE) {
    sprintf(buffer,"SEND_ONCE %s\n",command);
    
    if(send_packet(fd,buffer)==-1) {
      btperror("Could not send\ncommand");
      close(fd);
      return(EXIT_FAILURE);
    }
  } else {
    btperror("Input too long");
    close(fd);
    return(EXIT_FAILURE);
  }
  return(EXIT_SUCCESS);
}

int rcsend(char *command, int fd)
{
///     struct sockaddr_un addr;
	//      int fd;
	char buffer[PACKET_SIZE+1];
        char scratch[40];
	char remote[40];
	char macropart[40];
	int remote_set;
	char *scratchptr;
	char *cindex;
	struct sigaction act;
	int sendstatus;
/* use this */
//	progname=argv[0];
//      progname="btp";

	// Should this be run only once, too?

	remote_set = 0;
       	act.sa_handler=sigalrm;
       	sigemptyset(&act.sa_mask);
       	act.sa_flags=0;           /* we need EINTR */
       	sigaction(SIGALRM,&act,NULL);

	//       	addr.sun_family=AF_UNIX;
	//	strcpy(addr.sun_path,LIRCD);
	//	fd=socket(AF_UNIX,SOCK_STREAM,0);
	//	if(fd==-1)
	//	{
	//	   btperror("Could not open\nsocket\n\nIs lircd running?");
	//	   // I know, but I really don't like applications just dying.
	//	   // 		   perror(progname);
	//	   // 		   exit(EXIT_FAILURE);
	//	   return(EXIT_FAILURE);
	//	};
	//	if(connect(fd,(struct sockaddr *)&addr,sizeof(addr))==-1)
	//	{
	//	  btperror("Could not connect\nto socket\n\nIs lircd running?");
	//	   // Same goes here.
	//	   //		   perror(progname);
	//	   //		   exit(EXIT_FAILURE);
	//	   return(EXIT_FAILURE);
	//	};

	if(!strncasecmp(command, "macro ", 6)) {
	  //Ok... I really hate string juggling.
	  //I miss those neat array tossing and automatic enumerator thingies.
	  //Lessee, split on comma, then colon. Or something like that.
	  strcpy(buffer, command);
	  scratchptr = strtok(buffer, " "); //Skip "macro ", init strtok.
	  while ((scratchptr = strtok(NULL, ",\n"))) {
	    strcpy(scratch, scratchptr);
	    if ((cindex = index(scratch, ':'))) {
	      cindex[0] = 0;
	      if (!strncasecmp(scratch, ".sleep", 6)) {
		sleep(atoi(cindex+1));
	      } else if (!strncasecmp(scratch, ".tenths", 7)) {
		usleep(atoi(cindex+1)*100000);
	      } else if (!strncasecmp(scratch, ".exit", 5)) {
		printf("%s\n", cindex+1);
		exit(EXIT_SUCCESS); //There must be a better way.
	      } else { //Not a command, but a remote.
		strcpy(remote, scratch);
		remote_set = 1;
		sprintf(macropart,"%s %s", scratch, cindex+1);
		sendstatus = valsend(fd, macropart);
		if(sendstatus == EXIT_FAILURE) {
		  return(EXIT_FAILURE);
		}
	      }
	    } else {
	      if(remote_set) {
		sprintf(macropart,"%s %s", remote, scratch);
		sendstatus = valsend(fd, macropart);
		if(sendstatus == EXIT_FAILURE) {
		  return(EXIT_FAILURE);
		}
	      } else {
		btperror("Macro error!\nMissing remote\ndefinition.");
		return(EXIT_FAILURE);
	      }
	    }
	  }
	  //Phew! Macros! And delays!
	} else {
	  sendstatus = valsend(fd, command);
	}
	//	close(fd);
	return(sendstatus);
}
